@@ -6,14 +6,13 @@ A basic wrapper for the Discord API.
 
 :copyright: (c) 2015-2021 Rapptz & (c) 2021-present Pycord Development
 :license: MIT, see LICENSE for more details.
-
 """
 
 __title__ = "pycord"
 __author__ = "Pycord Development"
 __license__ = "MIT"
 __copyright__ = "Copyright 2015-2021 Rapptz & Copyright 2021-present Pycord Development"
-__version__ = "2.0.1"
+__version__ = "2.2.2"
 
 __path__ = __import__("pkgutil").extend_path(__path__, __name__)
 
@@ -25,6 +24,7 @@ from .activity import *
 from .appinfo import *
 from .asset import *
 from .audit_logs import *
+from .automod import *
 from .bot import *
 from .channel import *
 from .client import *
@@ -75,6 +75,8 @@ class VersionInfo(NamedTuple):
     serial: int
 
 
-version_info: VersionInfo = VersionInfo(major=2, minor=0, micro=1, releaselevel="final", serial=0)
+version_info: VersionInfo = VersionInfo(
+    major=2, minor=2, micro=2, releaselevel="final", serial=0
+)
 
 logging.getLogger(__name__).addHandler(logging.NullHandler())
