@@ -2,19 +2,17 @@
 Contains definitions for message classes that are text based."""
 
 
-from pickle import GLOBAL
 from typing import Any, Dict, List, Iterable, Optional, Union, Literal
 from datetime import datetime, timedelta
 from typeguard import typechecked
 
 from .base import *
 from ..dtypes import *
-from ..tracing import *
-from ..common import *
+from ..logging.tracing import *
 from ..exceptions import *
 
 from .. import client
-from .. import sql
+from ..logging import sql
 from .. import misc
 
 import asyncio
@@ -26,8 +24,18 @@ __all__ = (
     "DirectMESSAGE"
 )
 
-@typechecked
-@sql._register_type("MessageTYPE")
+# Configuration
+# ---------------------------
+RLIM_USER_WAIT_TIME = 20
+
+
+# Register message modes
+sql.register_type("MessageMODE", "send")(None)
+sql.register_type("MessageMODE", "edit")(None)
+sql.register_type("MessageMODE", "clear-send")(None)
+
+@misc.doc_category("Messages", path="message")
+@sql.register_type("MessageTYPE")
 class TextMESSAGE(BaseMESSAGE):
     """
     This class is used for creating objects that represent messages which will be sent to Discord's TEXT CHANNELS.
@@ -59,20 +67,21 @@ class TextMESSAGE(BaseMESSAGE):
             :caption: **Randomized** sending period between **5** seconds and **10** seconds.
 
             # Time between each send is somewhere between 5 seconds and 10 seconds.
-            daf.TextMESSAGE(start_period=timedelta(5), end_period=timedelta(10), data="Second Message", channels=[12345], mode="send", start_in=timedelta(seconds=0))
+            daf.TextMESSAGE(start_period=timedelta(seconds=5), end_period=timedelta(seconds=10), data="Second Message", channels=[12345], mode="send", start_in=timedelta(seconds=0))
 
         .. code-block:: python
             :caption: **Fixed** sending period at **10** seconds
 
             # Time between each send is exactly 10 seconds.
-            daf.TextMESSAGE(start_period=None, end_period=timedelta(10), data="Second Message", channels=[12345], mode="send", start_in=timedelta(seconds=0))
+            daf.TextMESSAGE(start_period=None, end_period=timedelta(seconds=10), data="Second Message", channels=[12345], mode="send", start_in=timedelta(seconds=0))
 
-    data: Union[str, EMBED, FILE, List[Union[str, EMBED, FILE]], _FunctionBaseCLASS]
+    data: Union[str, EMBED, discord.Embed, FILE, List[Union[str, EMBED, discord.Embed, FILE]], _FunctionBaseCLASS]
         The data parameter is the actual data that will be sent using discord's API. The data types of this parameter can be:
             - str (normal text),
-            - :ref:`EMBED`,
+            - :ref:`EMBED`
+            - :class:`discord.Embed`,
             - :ref:`FILE`,
-            - List/Tuple containing any of the above arguments (There can up to 1 string, up to 1 :ref:`EMBED` and up to 10 :ref:`FILE` objects. If more than 1 string or embeds are sent, the framework will only consider the last found).
+            - List/Tuple containing any of the above arguments (There can up to 1 string, up to 1 :ref:`EMBED` / :class:`discord.Embed` and up to 10 :ref:`FILE` objects. If more than 1 string or embeds are sent, the framework will only consider the last found).
             - Function that accepts any amount of parameters and returns any of the above types. To pass a function, YOU MUST USE THE :ref:`data_function` decorator on the function before passing the function to the daf.
 
     channels: Iterable[Union[int, discord.TextChannel, discord.Thread]]
@@ -102,10 +111,11 @@ class TextMESSAGE(BaseMESSAGE):
     )
     __logname__: str = "TextMESSAGE"               # Used for registering SQL types and to get the message type for saving the log
 
+    @typechecked
     def __init__(self, 
                  start_period: Union[int, timedelta, None],
                  end_period: Union[int, timedelta],
-                 data: Union[str, EMBED, FILE, Iterable[Union[str, EMBED, FILE]], _FunctionBaseCLASS],
+                 data: Union[str, EMBED, discord.Embed, FILE, Iterable[Union[str, EMBED, discord.Embed, FILE]], _FunctionBaseCLASS],
                  channels: Iterable[Union[int, discord.TextChannel, discord.Thread]],
                  mode: Literal["send", "edit", "clear-send"] = "send",
                  start_in: Union[timedelta, bool]=timedelta(seconds=0),
@@ -140,9 +150,9 @@ class TextMESSAGE(BaseMESSAGE):
                     reason.code in {50007, 10003}):     # Not Forbidden, but bad error codes
                     self.channels.remove(channel)
 
-    def _generate_log_context(self,
+    def generate_log_context(self,
                              text: Optional[str],
-                             embed: Optional[EMBED],
+                             embed: Union[discord.Embed, EMBED],
                              files: List[FILE],
                              succeeded_ch: List[Union[discord.TextChannel, discord.Thread]],
                              failed_ch: List[Dict[str, Any]]) -> Dict[str, Any]:
@@ -153,7 +163,7 @@ class TextMESSAGE(BaseMESSAGE):
         -----------
         text: str
             The text that was sent.
-        embed: EMBED
+        embed: Union[discord.Embed, EMBED]
             The embed that was sent.
         files: List[FILE]
             List of files that were sent.
@@ -165,22 +175,33 @@ class TextMESSAGE(BaseMESSAGE):
         Returns
         ----------
         Dict[str, Any]
-            Dictionary containing:
-
-                - sent_data: Dict[str, Any]:
-                    - text: str
-                        The text that was sent.
-                    - embed: Dict[str, Any]
-                        The embed that was sent.
-                    - files: List[str]
-                        List of files that were sent.
-
-                - channels: Dict[str, List]:
-                    - successful: List[Dict[str, int]] - List of dictionaries containing name of the channel and snowflake id of the channels.
-                    - failed: List[Dict[str, Any]] - List of dictionaries containing name of the channel (str), snowflake id (int) and reason why streaming to channel failed (str).
-
-                - type: str - The type of the message, this is always TextMESSAGE.
-                - mode: str - The mode used to send the message (send, edit, clear-send).
+            .. code-block:: python
+                
+                {
+                    sent_data:
+                    {
+                        text: str - The text that was sent,
+                        embed: Dict[str, Any] - The embed that was sent,
+                        files: List[str] - List of files that were sent
+                    },
+                    
+                    channels:
+                    {
+                        successful:
+                        {
+                            id: int - Snowflake id,
+                            name: str - Channel name
+                        },
+                        failed:
+                        {
+                            id: int - Snowflake id,
+                            name: str - Channel name,
+                            reason: str - Exception that caused the error
+                        }
+                    },
+                    type: str - The type of the message, this is always TextMESSAGE,
+                    mode: str - The mode used to send the message (send, edit, clear-send)
+                }
         """
 
         succeeded_ch = [{"name": str(channel), "id" : channel.id} for channel in succeeded_ch]
@@ -222,7 +243,7 @@ class TextMESSAGE(BaseMESSAGE):
             for element in data:
                 if isinstance(element, str):
                     _data_to_send["text"] = element
-                elif isinstance(element, EMBED):
+                elif isinstance(element, (EMBED, discord.Embed)):
                     _data_to_send["embed"] = element
                 elif isinstance(element, FILE):
                     _data_to_send["files"].append(element)
@@ -300,7 +321,7 @@ class TextMESSAGE(BaseMESSAGE):
     async def _send_channel(self,
                            channel: Union[discord.TextChannel, discord.Thread, None],
                            text: Optional[str],
-                           embed: Optional[EMBED],
+                           embed: Optional[Union[discord.Embed, EMBED]],
                            files: List[FILE]) -> dict:
         """
         Sends data to specific channel
@@ -315,7 +336,7 @@ class TextMESSAGE(BaseMESSAGE):
             The channel in which to send the data.
         text: str
             The text to send.
-        embed: EMBED
+        embed: Union[discord.Embed, EMBED]
             The embedded frame to send.
         files: List[FILE]
             List of files to send.
@@ -341,11 +362,10 @@ class TextMESSAGE(BaseMESSAGE):
                 if (self.mode in {"send" , "clear-send"} or # Mode dictates to send new message or delete previous and then send new message or mode dictates edit but message was  never sent to this channel before
                     self.mode == "edit" and self.sent_messages.get(channel.id, None) is None
                 ):
-                    discord_sent_msg = await channel.send(  text,
+                    self.sent_messages[channel.id] = await channel.send(  text,
                                                             embed=embed,
                                                             # Create discord.File objects here so it is catched by the except block and then logged
                                                             files=[discord.File(fwFILE.filename) for fwFILE in files])
-                    self.sent_messages[channel.id] = discord_sent_msg
                 # Mode is edit and message was already send to this channel
                 elif self.mode == "edit":
                     await self.sent_messages[channel.id].edit ( text,
@@ -365,7 +385,7 @@ class TextMESSAGE(BaseMESSAGE):
         Returns
         ----------
         Union[Dict, None]
-            Returns a dictionary generated by the ``_generate_log_context`` method or the None object if message wasn't ready to be sent (:ref:`data_function` returned None or an invalid type)
+            Returns a dictionary generated by the ``generate_log_context`` method or the None object if message wasn't ready to be sent (:ref:`data_function` returned None or an invalid type)
 
             This is then passed to :ref:`GUILD`._generate_log method.
         """
@@ -385,10 +405,11 @@ class TextMESSAGE(BaseMESSAGE):
                     errored_channels.append({"channel":channel, "reason": context["reason"]})
 
             self._update_state(errored_channels)
-            return self._generate_log_context(**data_to_send, succeeded_ch=succeeded_channels, failed_ch=errored_channels) # Return sent data + failed and successful function for logging purposes
+            return self.generate_log_context(**data_to_send, succeeded_ch=succeeded_channels, failed_ch=errored_channels) # Return sent data + failed and successful function for logging purposes
 
         return None
 
+    @typechecked
     @misc._async_safe("update_semaphore")
     async def update(self, _init_options: Optional[dict] = {}, **kwargs: Any):
         """
@@ -416,13 +437,12 @@ class TextMESSAGE(BaseMESSAGE):
             kwargs["start_in"] = timedelta(seconds=0)
         
         if not len(_init_options):
-            _init_options = {"guild": self.parent.apiobject}
+            _init_options = {"parent": self.parent}
 
         await misc._update(self, init_options=_init_options, **kwargs) # No additional modifications are required
 
-
-@typechecked
-@sql._register_type("MessageTYPE")
+@misc.doc_category("Messages", path="message")
+@sql.register_type("MessageTYPE")
 class DirectMESSAGE(BaseMESSAGE):
     """
     This class is used for creating objects that represent messages which will be sent to Discord's TEXT CHANNELS.
@@ -454,20 +474,21 @@ class DirectMESSAGE(BaseMESSAGE):
             :caption: **Randomized** sending period between **5** seconds and **10** seconds.
 
             # Time between each send is somewhere between 5 seconds and 10 seconds.
-            daf.DirectMESSAGE(start_period=timedelta(5), end_period=timedelta(10), data="Second Message",  mode="send", start_in=timedelta(seconds=0))
+            daf.DirectMESSAGE(start_period=timedelta(seconds=5), end_period=timedelta(seconds=10), data="Second Message",  mode="send", start_in=timedelta(seconds=0))
 
         .. code-block:: python
             :caption: **Fixed** sending period at **10** seconds
 
             # Time between each send is exactly 10 seconds.
-            daf.DirectMESSAGE(start_period=None, end_period=timedelta(10), data="Second Message",  mode="send", start_in=timedelta(seconds=0))
+            daf.DirectMESSAGE(start_period=None, end_period=timedelta(seconds=10), data="Second Message",  mode="send", start_in=timedelta(seconds=0))
 
-    data: Union[str, EMBED, FILE, List[Union[str, EMBED, FILE]], _FunctionBaseCLASS]
+    data: Union[str, EMBED, discord.Embed FILE, List[Union[str, EMBED, discord.Embed, FILE]], _FunctionBaseCLASS]
         The data parameter is the actual data that will be sent using discord's API. The data types of this parameter can be:
             - str (normal text),
             - :ref:`EMBED`,
+            - :class:`discord.Embed`,
             - :ref:`FILE`,
-            - List/Tuple containing any of the above arguments (There can up to 1 string, up to 1 :ref:`EMBED` and up to 10 :ref:`FILE` objects. If more than 1 string or embeds are sent, the framework will only consider the last found).
+            - List/Tuple containing any of the above arguments (There can up to 1 string, up to 1 :ref:`EMBED` / :class:`discord.Embed` and up to 10 :ref:`FILE` objects. If more than 1 string or embeds are sent, the framework will only consider the last found).
             - Function that accepts any amount of parameters and returns any of the above types. To pass a function, YOU MUST USE THE :ref:`data_function` decorator on the function before passing the function to the daf.
 
     mode: str
@@ -495,10 +516,11 @@ class DirectMESSAGE(BaseMESSAGE):
     )
     __logname__ = "DirectMESSAGE"               # Used for logging (type key) and sql lookup table type registration
 
+    @typechecked
     def __init__(self,
                  start_period: Union[int, timedelta, None],
                  end_period: Union[int, timedelta],
-                 data: Union[str, EMBED, FILE, Iterable[Union[str, EMBED, FILE]], _FunctionBaseCLASS],
+                 data: Union[str, EMBED, discord.Embed, FILE, Iterable[Union[str, EMBED, discord.Embed, FILE]], _FunctionBaseCLASS],
                  mode: Literal["send", "edit", "clear-send"] = "send",
                  start_in: Union[timedelta, bool] = timedelta(seconds=0),
                  remove_after: Optional[Union[int, timedelta, datetime]]=None):
@@ -507,10 +529,10 @@ class DirectMESSAGE(BaseMESSAGE):
         self.dm_channel = None
         self.previous_message = None
     
-    def _generate_log_context(self,
+    def generate_log_context(self,
                               success_context: Dict[str, Union[bool, Optional[Exception]]],
                               text : Optional[str],
-                              embed : Optional[EMBED],
+                              embed : Optional[Union[discord.Embed, EMBED]],
                               files : List[FILE]) -> Dict[str, Any]:
         """
         Generates information about the message send attempt that is to be saved into a log.
@@ -519,7 +541,7 @@ class DirectMESSAGE(BaseMESSAGE):
         -----------
         text: str
             The text that was sent.
-        embed: EMBED
+        embed: Union[discord.Embed, EMBED]
             The embed that was sent.
         files: List[FILE]
             List of files that were sent.
@@ -531,19 +553,23 @@ class DirectMESSAGE(BaseMESSAGE):
         Returns
         ----------
         Dict[str, Any]
-            Dictionary containing:
+            .. code-block:: python
 
-                - sent_data: Dict[str, str]:
-                    - streamed_audio - The filename that was streamed/youtube url
-
-                - success_info: Dict[str, Any]:
-                    - success: bool - Was sending successful or not
-                    - reason:  str  - If it was unsuccessful, what was the reason
-                    - delete: bool  - Signals the guild object to remove this message from the list
-                                      due to unrecoverable error (if set to True).
-
-                - type: str - The type of the message, this is always TextMESSAGE.
-                - mode: str - The mode used to send the message (send, edit, clear-send).
+                {
+                    sent_data:
+                    {
+                        text: str - The text that was sent,
+                        embed: Dict[str, Any] - The embed that was sent,
+                        files: List[str] - List of files that were sent.
+                    },
+                    success_info:
+                    {
+                        success: bool - Was sending successful or not,
+                        reason:  str  - If it was unsuccessful, what was the reason
+                    },
+                    type: str - The type of the message, this is always DirectMESSAGE,
+                    mode: str - The mode used to send the message (send, edit, clear-send)
+                }
         """
         embed = embed.to_dict() if embed is not None else None
         files = [x.filename for x in files]
@@ -631,13 +657,13 @@ class DirectMESSAGE(BaseMESSAGE):
                         self.parent.remove_message(m)
 
             if ex.status in {400, 403}: # Bad Request
-                await asyncio.sleep(RLIM_USER_WAIT_TIME * 5) # To avoid triggering self-bot detection
+                await asyncio.sleep(RLIM_USER_WAIT_TIME) # To avoid triggering self-bot detection
 
         return handled
 
     async def _send_channel(self,
                            text: Optional[str],
-                           embed: Optional[EMBED],
+                           embed: Optional[Union[discord.Embed, EMBED]],
                            files: List[FILE]) -> dict:
         """
         Sends data to the DM channel (user).
@@ -680,7 +706,7 @@ class DirectMESSAGE(BaseMESSAGE):
         Returns
         ----------
         Union[Dict, None]
-            Returns a dictionary generated by the ``_generate_log_context`` method or the None object if message wasn't ready to be sent (:ref:`data_function` returned None or an invalid type)
+            Returns a dictionary generated by the ``generate_log_context`` method or the None object if message wasn't ready to be sent (:ref:`data_function` returned None or an invalid type)
 
             This is then passed to :ref:`GUILD`._generate_log method.
         """
@@ -689,10 +715,11 @@ class DirectMESSAGE(BaseMESSAGE):
         if any(data_to_send.values()):            
             context = await self._send_channel(**data_to_send)
             self._update_state()
-            return self._generate_log_context(context, **data_to_send)
+            return self.generate_log_context(context, **data_to_send)
 
         return None
 
+    @typechecked
     @misc._async_safe("update_semaphore")
     async def update(self, _init_options: Optional[dict] = {}, **kwargs):
         """
@@ -721,6 +748,6 @@ class DirectMESSAGE(BaseMESSAGE):
             kwargs["start_in"] = timedelta(seconds=0)
 
         if not len(_init_options):
-            _init_options = {"user" : self.parent.apiobject}
+            _init_options = {"parent" : self.parent}
 
         await misc._update(self, init_options=_init_options, **kwargs) # No additional modifications are required
